from typing import get_type_hints

from graphql import GraphQLField

from .constants import IS_STRAWBERRY_FIELD
from .exceptions import MissingArgumentsAnnotationsError, MissingReturnAnnotationError
from .type_converter import get_graphql_type_for_annotation
from .utils.inspect import get_func_args


def field(wrap):
    setattr(wrap, IS_STRAWBERRY_FIELD, True)
    annotations = get_type_hints(wrap)

    name = wrap.__name__

    if "return" not in annotations:
        raise MissingReturnAnnotationError(name)

    field_type = get_graphql_type_for_annotation(annotations["return"], name)

    function_arguments = set(get_func_args(wrap)) - {"self", "info"}

    arguments_annotations = {
        key: value
        for key, value in annotations.items()
        if key not in ["info", "return"]
    }

    annotated_function_arguments = set(arguments_annotations.keys())
    arguments_missing_annotations = function_arguments - annotated_function_arguments

    if len(arguments_missing_annotations) > 0:
        raise MissingArgumentsAnnotationsError(name, arguments_missing_annotations)

    arguments = {
        name: get_graphql_type_for_annotation(annotation, name)
        for name, annotation in arguments_annotations.items()
    }

    wrap.field = GraphQLField(field_type, args=arguments, resolve=wrap)
    return wrap



import click
import sys

import os
from starlette.applications import Starlette
from starlette.responses import HTMLResponse
import importlib

import uvicorn

import hupper

from strawberry.contrib.starlette import GraphQLApp


@click.group()
def run():
    pass


@run.command("server")
@click.argument("module", type=str)
def server(module):
    sys.path.append(os.getcwd())

    reloader = hupper.start_reloader("strawberry.cli.run")

    schema_module = importlib.import_module(module)

    reloader.watch_files([schema_module.__file__])

    app = Starlette(debug=True)
    app.add_route("/graphql", GraphQLApp(schema_module.schema))
    uvicorn.run(app, host="0.0.0.0", port=8000, log_level="error")



import typing

from dataclasses import dataclass
from graphql import GraphQLField, GraphQLObjectType
from graphql.utilities.schema_printer import print_type

from .constants import IS_STRAWBERRY_FIELD
from .type_converter import get_graphql_type_for_annotation


def _get_resolver(cls, field_name):
    def _resolver(obj, info):
        # TODO: can we make this nicer?
        # does it work in all the cases?

        field_resolver = getattr(cls(**(obj.__dict__ if obj else {})), field_name)

        if getattr(field_resolver, IS_STRAWBERRY_FIELD, False):
            return field_resolver(obj, info)

        return field_resolver

    return _resolver


def _get_fields(cls):
    cls_annotations = typing.get_type_hints(cls)

    fields = {
        key: GraphQLField(
            get_graphql_type_for_annotation(value, field_name=key),
            resolve=_get_resolver(cls, key),
        )
        for key, value in cls_annotations.items()
    }

    fields.update(
        {
            key: value.field
            for key, value in cls.__dict__.items()
            if getattr(value, IS_STRAWBERRY_FIELD, False)
        }
    )

    return fields


def type(cls):
    def wrap():
        def repr_(self):
            return print_type(self.field)

        setattr(cls, "__repr__", repr_)

        cls._fields = _get_fields(cls)
        cls.field = GraphQLObjectType(name=cls.__name__, fields=cls._fields)

        return dataclass(cls, repr=False)

    return wrap()



import datetime
import functools
import json
import pathlib
import typing

from graphql import graphql
from graphql.error import GraphQLError, format_error as format_graphql_error
from pygments import highlight, lexers
from pygments.formatters import Terminal256Formatter
from starlette import status
from starlette.background import BackgroundTasks
from starlette.concurrency import run_in_threadpool
from starlette.requests import Request
from starlette.responses import HTMLResponse, JSONResponse, PlainTextResponse, Response
from starlette.types import ASGIInstance, Receive, Scope, Send

from .utils.graphql_lexer import GraphqlLexer


def _get_playground_template(request_path: str):
    here = pathlib.Path(__file__).parent
    templates_path = here / "templates"

    with open(templates_path / "playground.html") as f:
        template = f.read()

    return template.replace("{{REQUEST_PATH}}", request_path)


class GraphQLApp:
    def __init__(self, schema, playground: bool = True) -> None:
        self.schema = schema
        self.playground = playground

    def __call__(self, scope: Scope) -> ASGIInstance:
        return functools.partial(self.asgi, scope=scope)

    async def asgi(self, receive: Receive, send: Send, scope: Scope) -> None:
        request = Request(scope, receive=receive)
        response = await self.handle_graphql(request)
        await response(receive, send)

    def _debug_log(
        self, operation_name: str, query: str, variables: typing.Dict["str", typing.Any]
    ):
        if operation_name == "IntrospectionQuery":
            return

        now = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")

        print(f"[{now}]: {operation_name or 'No operation name'}")
        print(highlight(query, GraphqlLexer(), Terminal256Formatter()))

        if variables:
            variables_json = json.dumps(variables, indent=4)

            print(highlight(variables_json, lexers.JsonLexer(), Terminal256Formatter()))

    async def handle_graphql(self, request: Request) -> Response:
        if request.method in ("GET", "HEAD"):
            if "text/html" in request.headers.get("Accept", ""):
                if not self.playground:
                    return PlainTextResponse(
                        "Not Found", status_code=status.HTTP_404_NOT_FOUND
                    )
            return await self.handle_playground(request)

        elif request.method == "POST":
            content_type = request.headers.get("Content-Type", "")

            if "application/json" in content_type:
                data = await request.json()
            elif "application/graphql" in content_type:
                body = await request.body()
                text = body.decode()
                data = {"query": text}
            elif "query" in request.query_params:
                data = request.query_params
            else:
                return PlainTextResponse(
                    "Unsupported Media Type",
                    status_code=status.HTTP_415_UNSUPPORTED_MEDIA_TYPE,
                )
        else:
            return PlainTextResponse(
                "Method Not Allowed", status_code=status.HTTP_405_METHOD_NOT_ALLOWED
            )

        try:
            query = data["query"]
            variables = data.get("variables")
            operation_name = data.get("operationName")
        except KeyError:
            return PlainTextResponse(
                "No GraphQL query found in the request",
                status_code=status.HTTP_400_BAD_REQUEST,
            )

        self._debug_log(operation_name, query, variables)

        background = BackgroundTasks()
        context = {"request": request, "background": background}

        result = await self.execute(
            query, variables=variables, context=context, operation_name=operation_name
        )
        error_data = (
            [format_graphql_error(err) for err in result.errors]
            if result.errors
            else None
        )
        response_data = {"data": result.data, "errors": error_data}
        status_code = (
            status.HTTP_400_BAD_REQUEST if result.errors else status.HTTP_200_OK
        )

        return JSONResponse(
            response_data, status_code=status_code, background=background
        )

    async def execute(self, query, variables=None, context=None, operation_name=None):
        return await graphql(
            self.schema,
            query,
            variable_values=variables,
            operation_name=operation_name,
            context_value=context,
        )

    async def handle_playground(self, request: Request) -> Response:
        text = _get_playground_template(request.url.path)

        return HTMLResponse(text)



from graphql import GraphQLSchema


# TODO: typings


class Schema(GraphQLSchema):
    def __init__(self, query):
        super().__init__(query=self._build_query(query))

    def _build_query(self, query):
        return query.field



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['strawberry',
 'strawberry.cli',
 'strawberry.contrib',
 'strawberry.contrib.starlette',
 'strawberry.contrib.starlette.utils',
 'strawberry.utils']

package_data = \
{'': ['*'], 'strawberry.contrib.starlette': ['templates/*']}

install_requires = \
['click>=7.0,<8.0',
 'graphql-core-next>=1.0,<2.0',
 'hupper>=1.5,<2.0',
 'pygments>=2.3,<3.0',
 'starlette>=0.11.3,<0.12.0',
 'uvicorn>=0.4.6,<0.5.0']

entry_points = \
{'console_scripts': ['strawberry = strawberry.cli:run']}

setup_kwargs = {
    'name': 'strawberry-graphql',
    'version': '0.3.1',
    'description': 'A library for creating GraphQL APIs',
    'long_description': None,
    'author': 'Patrick Arminio',
    'author_email': 'patrick.arminio@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
